@@ -4,34 +4,61 @@ use strict;
 use warnings;
 
 require Carp;
+require Scalar::Util;
 use Routes::Tiny::Pattern;
 
-our $VERSION = 0.1;
+our $VERSION = 0.14;
 
 sub new {
     my $class = shift;
+    my (%params) = @_;
 
-    my $self = {patterns => []};
+    my $self = {};
     bless $self, $class;
 
+    $self->{strict_trailing_slash} = $params{strict_trailing_slash};
+
+    $self->{parent_pattern}        = undef;
+    $self->{patterns}              = [];
+    $self->{names}                 = {};
+    $self->{strict_trailing_slash} = 1
+      unless defined $self->{strict_trailing_slash};
+
     return $self;
 }
 
 sub add_route {
-    my $self    = shift;
-    my $pattern = shift;
+    my $self = shift;
+    my ($pattern, @args) = @_;
 
-    $pattern = $self->_build_pattern(pattern => $pattern, @_);
+    $pattern = $self->_build_pattern(
+        strict_trailing_slash => $self->{strict_trailing_slash},
+        routes                => $self,
+        pattern               => $pattern,
+        @args
+    );
 
     push @{$self->{patterns}}, $pattern;
 
+    $self->_register_pattern_name($pattern) if $pattern->{name};
+
+    return $pattern;
+}
+
+sub mount {
+    my $self = shift;
+    my ($pattern, $routes, @args) = @_;
+
+    $pattern = $self->add_route($pattern, subroutes => $routes, @args);
+    $routes->{parent_pattern} = $pattern;
+    $self->_register_pattern_name($_) for values %{ $routes->{names} };
+    Scalar::Util::weaken($routes->{parent_pattern});
     return $pattern;
 }
 
 sub match {
     my $self = shift;
-    my $path = shift;
-    my @args = @_;
+    my ($path, @args) = @_;
 
     foreach my $pattern (@{$self->{patterns}}) {
         if (my $m = $pattern->match($path, @args)) {
@@ -44,24 +71,30 @@ sub match {
 
 sub build_path {
     my $self = shift;
-    my $name = shift;
+    my ($name, @args) = @_;
 
-    my $pattern = $self->_find_route($name);
+    my $pattern = $self->{names}->{$name};
 
-    return $pattern->build_path(@_) if $pattern;
+    return $pattern->build_path(@args) if $pattern;
 
     Carp::croak("Unknown name '$name' used to build a path");
 }
 
-sub _find_route {
+sub _register_pattern_name {
     my $self = shift;
-    my ($name) = @_;
+    my ($pattern) = @_;
 
-    foreach my $pattern (@{$self->{patterns}}) {
-        return $pattern if $pattern->name && $pattern->name eq $name;
+    my $name = $pattern->name;
+    if (exists $self->{names}->{ $name }) {
+        Carp::carp("pattern name '$name' already used");
+    }
+    else {
+        $self->{names}->{ $name } = $pattern;
+        my $parent_routes = $self->{parent_pattern} && $self->{parent_pattern}->{routes};
+        if ($parent_routes) {
+            $parent_routes->_register_pattern_name(@_);
+        }
     }
-
-    return;
 }
 
 sub _build_pattern { shift; return Routes::Tiny::Pattern->new(@_) }
@@ -104,6 +137,11 @@ Routes::Tiny - Routes
     # Matching with method
     my $match = $routes->match('/hello/world', method => 'GET');
 
+    # Subroutes
+    my $subroutes = Routes::Tiny->new;
+    $subroutes->add_route('/article/:id');
+    $routes->mount('/admin/', $subroutes);
+
 =head1 DESCRIPTION
 
 L<Routes::Tiny> is a lightweight routes implementation.
@@ -176,11 +214,37 @@ It is possible to pass arguments to the match object AS IS.
 
 It is possible to reconstruct a path from route's name and parameters.
 
+=head2 C<Subroutes>
+
+    $subroutes = Routes::Tiny->new;
+    $subroutes->add_route('/articles/:id', name => 'admin-article');
+    $routes->mount('/admin/', $subroutes);
+
+    $match = $routes->match('/admin/articles/3/');
+    # $match->captures is {id => 3}
+
+It is possible to capture params in mount routes
+
+    $subroutes = Routes::Tiny->new;
+    $subroutes->add_route('/comments/:page/', name => 'comments');
+    $routes->mount('/:type/:id/', $subroutes);
+
+    $match = $routes->match('/articles/3/comments/5/');
+    # $match->captures is {page => 5}
+    # $match->parent->captures is {type => 'articles', id => 3}
+
+Parent routes mounts names of children routes, so it's possible to buil path
+
+    $path = $routes->build_path('admin-article', id => 123);
+    # $path is '/admin/articles/123'
+    $path = $routes->build_path('comments', type => 'articles', id => 123, page => 5);
+    # $path is '/articles/123/comments/5/'
+
 =head1 WARNINGS
 
 =head2 C<Trailing slash issue>
 
-Trailing slash is important. Maybe this will be changed in the future.
+Trailing slash is important.
 
     $routes->add_route('/articles');
 
@@ -188,6 +252,10 @@ Trailing slash is important. Maybe this will be changed in the future.
 
     $routes->add_route('/articles/');
 
+If you don't want this behaviour pass C<strict_trailing_slash> to the constructor:
+
+    my $routes = Routes::Tiny->new(strict_trailing_slash => 0);
+
 =head1 METHODS
 
 =head2 C<new>
@@ -200,6 +268,12 @@ Trailing slash is important. Maybe this will be changed in the future.
 
 Add a new route.
 
+=head2 C<mount>
+
+    $routes->mount('/admin/', $subroutes)
+
+Includes one Routes::Tiny instance into another with given prefix.
+
 =head2 C<match>
 
     $routes->match('/hello/world');
@@ -224,6 +298,10 @@ Sergey Zasenko (und3f)
 
 Roman Galeev (jamhed)
 
+Dmitry Smal (mialinx)
+
+Dinar (ziontab)
+
 =head1 AUTHOR
 
 Viacheslav Tykhanovskyi, C<vti@cpan.org>.
